@@ -2,12 +2,14 @@
 # --!-- coding: utf8 --!--
 import imp
 import os
+import re
 
-from PyQt5.QtCore import (pyqtSignal, QSignalMapper, QTimer, QSettings, Qt,
+from PyQt5.Qt import qVersion, PYQT_VERSION_STR
+from PyQt5.QtCore import (pyqtSignal, QSignalMapper, QTimer, QSettings, Qt, QPoint,
                           QRegExp, QUrl, QSize, QModelIndex)
 from PyQt5.QtGui import QStandardItemModel, QIcon, QColor
 from PyQt5.QtWidgets import QMainWindow, QHeaderView, qApp, QMenu, QActionGroup, QAction, QStyle, QListWidgetItem, \
-    QLabel, QDockWidget, QWidget
+    QLabel, QDockWidget, QWidget, QMessageBox
 
 from manuskript import settings
 from manuskript.enums import Character, PlotStep, Plot, World, Outline
@@ -34,15 +36,10 @@ from manuskript.ui.statusLabel import statusLabel
 
 # Spellcheck support
 from manuskript.ui.views.textEditView import textEditView
-
-try:
-    import enchant
-except ImportError:
-    enchant = None
-
+from manuskript.functions import Spellchecker
 
 class MainWindow(QMainWindow, Ui_MainWindow):
-    dictChanged = pyqtSignal(str)
+    # dictChanged = pyqtSignal(str)
 
     # Tab indexes
     TabInfos = 0
@@ -62,9 +59,10 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 
         # Var
         self.currentProject = None
+        self.projectDirty = None  # has the user made any unsaved changes ?
         self._lastFocus = None
         self._lastMDEditView = None
-        self._defaultCursorFlashTime = 1000 # Overriden at startup with system
+        self._defaultCursorFlashTime = 1000 # Overridden at startup with system
                                             # value. In manuskript.main.
         self._autoLoadProject = None  # Used to load a command line project
 
@@ -278,6 +276,15 @@ class MainWindow(QMainWindow, Ui_MainWindow):
                 break
             new = new.parent()
 
+    def projectName(self):
+        """
+        Returns a user-friendly name for the loaded project.
+        """
+        pName = os.path.split(self.currentProject)[1]
+        if pName.endswith('.msk'):
+            pName=pName[:-4]
+        return pName
+
     ###############################################################################
     # SUMMARY
     ###############################################################################
@@ -623,38 +630,76 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         # We force to emit even if it opens on the current tab
         self.tabMain.currentChanged.emit(settings.lastTab)
 
+        # Make sure we can update the window title later.
+        self.currentProject = project
+        self.projectDirty = False
+        QSettings().setValue("lastProject", project)
+
         # Add project name to Window's name
-        pName = os.path.split(project)[1]
-        if pName.endswith('.msk'):
-            pName=pName[:-4]
-        self.setWindowTitle(pName + " - " + self.tr("Manuskript"))
+        self.setWindowTitle(self.projectName() + " - " + self.tr("Manuskript"))
 
         # Stuff
         # self.checkPersosID()  # Shouldn't be necessary any longer
 
-        self.currentProject = project
-        QSettings().setValue("lastProject", project)
-
         # Show main Window
         self.switchToProject()
 
+    def handleUnsavedChanges(self):
+        """
+        There may be some currently unsaved changes, but the action the user triggered
+        will result in the project or application being closed. To save, or not to save?
+
+        Or just bail out entirely?
+
+        Sometimes it is best to just ask.
+        """
+
+        if not self.projectDirty:
+            return True  # no unsaved changes, all is good
+
+        msg = QMessageBox(QMessageBox.Question,
+            self.tr("Save project?"),
+            "<p><b>" +
+                self.tr("Save changes to project \"{}\" before closing?").format(self.projectName()) +
+            "</b></p>" +
+            "<p>" +
+                self.tr("Your changes will be lost if you don't save them.") +
+            "</p>",
+            QMessageBox.Save | QMessageBox.Discard | QMessageBox.Cancel)
+
+        ret = msg.exec()
+
+        if ret == QMessageBox.Cancel:
+            return False  # the situation has not been handled, cancel action
+
+        if ret == QMessageBox.Save:
+            self.saveDatas()
+
+        return True  # the situation has been handled
+
+
     def closeProject(self):
 
         if not self.currentProject:
             return
 
+        # Make sure data is saved.
+        if (self.projectDirty and settings.saveOnQuit == True):
+             self.saveDatas()
+        elif not self.handleUnsavedChanges():
+             return  # user cancelled action
+
         # Close open tabs in editor
         self.mainEditor.closeAllTabs()
 
-        # Save datas
-        self.saveDatas()
-
         self.currentProject = None
+        self.projectDirty = None
         QSettings().setValue("lastProject", "")
 
         # Clear datas
         self.loadEmptyDatas()
         self.saveTimer.stop()
+        self.saveTimerNoChanges.stop()
         loadSave.clearSaveCache()
 
         self.breakConnections()
@@ -716,23 +761,6 @@ class MainWindow(QMainWindow, Ui_MainWindow):
             self._toolbarState = ""
 
     def closeEvent(self, event):
-        # Save State and geometry and other things
-        sttgns = QSettings(qApp.organizationName(), qApp.applicationName())
-        sttgns.setValue("geometry", self.saveGeometry())
-        sttgns.setValue("windowState", self.saveState())
-        sttgns.setValue("metadataState", self.redacMetadata.saveState())
-        sttgns.setValue("revisionsState", self.redacMetadata.revisions.saveState())
-        sttgns.setValue("splitterRedacH", self.splitterRedacH.saveState())
-        sttgns.setValue("splitterRedacV", self.splitterRedacV.saveState())
-        sttgns.setValue("toolbar", self.toolbar.saveState())
-
-        # If we are not in the welcome window, we update the visibility
-        # of the docks widgets
-        if self.stack.currentIndex() == 1:
-            self.updateDockVisibility()
-        # Storing the visibility of docks to restore it on restart
-        sttgns.setValue("docks", self._dckVisibility)
-
         # Specific settings to save before quitting
         settings.lastTab = self.tabMain.currentIndex()
 
@@ -740,14 +768,42 @@ class MainWindow(QMainWindow, Ui_MainWindow):
             # Remembering the current items (stores outlineItem's ID)
             settings.openIndexes = self.mainEditor.tabSplitter.openIndexes()
 
-        # Save data from models
-        if self.currentProject and settings.saveOnQuit:
-            self.saveDatas()
+            # Save data from models
+            if settings.saveOnQuit:
+                self.saveDatas()
+            elif not self.handleUnsavedChanges():
+                event.ignore() # user opted to cancel the close action
 
             # closeEvent
             # QMainWindow.closeEvent(self, event)  # Causing segfaults?
 
+        # User may have canceled close event, so make sure we indeed want to close.
+        # This is necessary because self.updateDockVisibility() hides UI elements.
+        if event.isAccepted():
+            # Save State and geometry and other things
+            appSettings = QSettings(qApp.organizationName(), qApp.applicationName())
+            appSettings.setValue("geometry", self.saveGeometry())
+            appSettings.setValue("windowState", self.saveState())
+            appSettings.setValue("metadataState", self.redacMetadata.saveState())
+            appSettings.setValue("revisionsState", self.redacMetadata.revisions.saveState())
+            appSettings.setValue("splitterRedacH", self.splitterRedacH.saveState())
+            appSettings.setValue("splitterRedacV", self.splitterRedacV.saveState())
+            appSettings.setValue("toolbar", self.toolbar.saveState())
+
+            # If we are not in the welcome window, we update the visibility
+            # of the docks widgets
+            if self.stack.currentIndex() == 1:
+                self.updateDockVisibility()
+
+            # Storing the visibility of docks to restore it on restart
+            appSettings.setValue("docks", self._dckVisibility)
+
     def startTimerNoChanges(self):
+        """
+        Something changed in the project that requires auto-saving.
+        """
+        self.projectDirty = True
+
         if settings.autoSaveNoChanges:
             self.saveTimerNoChanges.start()
 
@@ -762,11 +818,24 @@ class MainWindow(QMainWindow, Ui_MainWindow):
             self.currentProject = projectName
             QSettings().setValue("lastProject", projectName)
 
-        r = loadSave.saveProject()  # version=0
+        # Stop the timer before saving: if auto-saving fails (bugs out?) we don't want it
+        # to keep trying and continuously hitting the failure condition. Nor do we want to
+        # risk a scenario where the timer somehow triggers a new save while saving.
         self.saveTimerNoChanges.stop()
 
+        if self.currentProject is None:
+            # No UI feedback here as this code path indicates a race condition that happens
+            # after the user has already closed the project through some way. But in that
+            # scenario, this code should not be reachable to begin with.
+            print("Bug: there is no current project to save.")
+            return
+
+        r = loadSave.saveProject()  # version=0
+
         projectName = os.path.basename(self.currentProject)
         if r:
+            self.projectDirty = False  # successful save, clear dirty flag
+
             feedback = self.tr("Project {} saved.").format(projectName)
             F.statusMessage(feedback, importance=0)
         else:
@@ -1073,14 +1142,17 @@ class MainWindow(QMainWindow, Ui_MainWindow):
     # HELP
     ###############################################################################
 
+    def centerChildWindow(self, win):
+        r = win.geometry()
+        r2 = self.geometry()
+        win.move(r2.center() - QPoint(r.width()/2, r.height()/2))
+
     def about(self):
         self.dialog = aboutDialog(mw=self)
         self.dialog.setFixedSize(self.dialog.size())
         self.dialog.show()
         # Center about dialog
-        r = self.dialog.geometry()
-        r2 = self.geometry()
-        self.dialog.move(r2.center() - r.center())
+        self.centerChildWindow(self.dialog)
 
     ###############################################################################
     # GENERAL AKA UNSORTED
@@ -1246,24 +1318,29 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         self.actShowHelp.setChecked(False)
 
         # Spellcheck
-        if enchant:
+        if Spellchecker.isInstalled():
             self.menuDict = QMenu(self.tr("Dictionary"))
             self.menuDictGroup = QActionGroup(self)
             self.updateMenuDict()
             self.menuTools.addMenu(self.menuDict)
 
             self.actSpellcheck.toggled.connect(self.toggleSpellcheck, F.AUC)
-            self.dictChanged.connect(self.mainEditor.setDict, F.AUC)
-            self.dictChanged.connect(self.redacMetadata.setDict, F.AUC)
-            self.dictChanged.connect(self.outlineItemEditor.setDict, F.AUC)
+            # self.dictChanged.connect(self.mainEditor.setDict, F.AUC)
+            # self.dictChanged.connect(self.redacMetadata.setDict, F.AUC)
+            # self.dictChanged.connect(self.outlineItemEditor.setDict, F.AUC)
 
         else:
             # No Spell check support
             self.actSpellcheck.setVisible(False)
-            a = QAction(self.tr("Install PyEnchant to use spellcheck"), self)
-            a.setIcon(self.style().standardIcon(QStyle.SP_MessageBoxWarning))
-            a.triggered.connect(self.openPyEnchantWebPage, F.AUC)
-            self.menuTools.addAction(a)
+            for lib, requirement in Spellchecker.supportedLibraries().items():
+                a = QAction(self.tr("Install {}{} to use spellcheck").format(lib, requirement or ""), self)
+                a.setIcon(self.style().standardIcon(QStyle.SP_MessageBoxWarning))
+                # Need to bound the lib argument otherwise the lambda uses the same lib value across all calls
+                def gen_slot_cb(l):
+                    return lambda: self.openSpellcheckWebPage(l)
+                a.triggered.connect(gen_slot_cb(lib), F.AUC)
+                self.menuTools.addAction(a)
+
 
     ###############################################################################
     # SPELLCHECK
@@ -1271,37 +1348,75 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 
     def updateMenuDict(self):
 
-        if not enchant:
+        if not Spellchecker.isInstalled():
             return
 
         self.menuDict.clear()
-        for i in enchant.list_dicts():
-            a = QAction(str(i[0]), self)
-            a.setCheckable(True)
-            if settings.dict is None:
-                settings.dict = enchant.get_default_language()
-            if str(i[0]) == settings.dict:
-                a.setChecked(True)
-            a.triggered.connect(self.setDictionary, F.AUC)
-            self.menuDictGroup.addAction(a)
+        dictionaries = Spellchecker.availableDictionaries()
+
+        # Set first run dictionary
+        if settings.dict is None:
+            settings.dict = Spellchecker.getDefaultDictionary()
+
+        # Check if project dict is unavailable on this machine
+        dict_available = False
+        for lib, dicts in dictionaries.items():
+            if dict_available:
+                break
+            for i in dicts:
+                if Spellchecker.normalizeDictName(lib, i) == settings.dict:
+                    dict_available = True
+                    break
+        # Reset dict to default one if it's unavailable
+        if not dict_available:
+            settings.dict = Spellchecker.getDefaultDictionary()
+
+        for lib, dicts in dictionaries.items():
+            if len(dicts) > 0:
+                a = QAction(lib, self)
+            else:
+                a = QAction(self.tr("{} has no installed dictionaries").format(lib), self)
+            a.setEnabled(False)
             self.menuDict.addAction(a)
+            for i in dicts:
+                a = QAction(i, self)
+                a.data = lib
+                a.setCheckable(True)
+                if Spellchecker.normalizeDictName(lib, i) == settings.dict:
+                    a.setChecked(True)
+                a.triggered.connect(self.setDictionary, F.AUC)
+                self.menuDictGroup.addAction(a)
+                self.menuDict.addAction(a)
+            self.menuDict.addSeparator()
+
+        # If a new dictionary was chosen, apply the change and re-enable spellcheck if it was enabled.
+        if not dict_available:
+            self.setDictionary()
+            self.toggleSpellcheck(settings.spellcheck)
+
+        for lib, requirement in Spellchecker.supportedLibraries().items():
+            if lib not in dictionaries:
+                a = QAction(self.tr("{}{} is not installed").format(lib, requirement or ""), self)
+                a.setEnabled(False)
+                self.menuDict.addAction(a)
+                self.menuDict.addSeparator()
 
     def setDictionary(self):
-        if not enchant:
+        if not Spellchecker.isInstalled():
             return
 
         for i in self.menuDictGroup.actions():
             if i.isChecked():
                 # self.dictChanged.emit(i.text().replace("&", ""))
-                settings.dict = i.text().replace("&", "")
+                settings.dict = Spellchecker.normalizeDictName(i.data, i.text().replace("&", ""))
 
                 # Find all textEditView from self, and toggle spellcheck
                 for w in self.findChildren(textEditView, QRegExp(".*"),
                                            Qt.FindChildrenRecursively):
                     w.setDict(settings.dict)
 
-    def openPyEnchantWebPage(self):
-        F.openURL("http://pythonhosted.org/pyenchant/")
+    def openSpellcheckWebPage(self, lib):
+        F.openURL(Spellchecker.getLibraryURL(lib))
 
     def toggleSpellcheck(self, val):
         settings.spellcheck = val
@@ -1326,9 +1441,7 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         self.sw.hide()
         self.sw.setWindowModality(Qt.ApplicationModal)
         self.sw.setWindowFlags(Qt.Dialog)
-        r = self.sw.geometry()
-        r2 = self.geometry()
-        self.sw.move(r2.center() - r.center())
+        self.centerChildWindow(self.sw)
         if tab:
             self.sw.setTab(tab)
         self.sw.show()
@@ -1340,6 +1453,7 @@ class MainWindow(QMainWindow, Ui_MainWindow):
     def frequencyAnalyzer(self):
         self.fw = frequencyAnalyzer(self)
         self.fw.show()
+        self.centerChildWindow(self.fw)
 
     ###############################################################################
     # VIEW MENU
@@ -1468,17 +1582,41 @@ class MainWindow(QMainWindow, Ui_MainWindow):
     ###############################################################################
 
     def doImport(self):
+        # Warn about buggy Qt versions and import crash
+        #
+        # (Py)Qt 5.11 and 5.12 have a bug that can cause crashes when simply
+        # setting up various UI elements.
+        # This has been reported and verified to happen with File -> Import.
+        # See PR #611.
+        if re.match("^5\\.1[12](\\.?|$)", qVersion()):
+            warning1 = self.tr("PyQt / Qt versions 5.11 and 5.12 are known to cause a crash which might result in a loss of data.")
+            warning2 = self.tr("PyQt {} and Qt {} are in use.").format(qVersion(), PYQT_VERSION_STR)
+
+            # Don't translate for debug log.
+            print("WARNING:", warning1, warning2)
+
+            msg = QMessageBox(QMessageBox.Warning,
+                self.tr("Proceeding might crash and lose data"),
+                "<p><b>" +
+                    warning1 +
+                "</b></p>" +
+                "<p>" +
+                    warning2 +
+                "</p>",
+                QMessageBox.Abort | QMessageBox.Ignore)
+            msg.setDefaultButton(QMessageBox.Abort)
+
+            # Return because user heeds warning
+            if msg.exec() == QMessageBox.Abort:
+                return
+
+        # Proceed with Import
         self.dialog = importerDialog(mw=self)
         self.dialog.show()
+        self.centerChildWindow(self.dialog)
 
-        r = self.dialog.geometry()
-        r2 = self.geometry()
-        self.dialog.move(r2.center() - r.center())
 
     def doCompile(self):
         self.dialog = exporterDialog(mw=self)
         self.dialog.show()
-
-        r = self.dialog.geometry()
-        r2 = self.geometry()
-        self.dialog.move(r2.center() - r.center())
+        self.centerChildWindow(self.dialog)
