@@ -26,11 +26,6 @@ from manuskript.ui.views.textEditView import textEditView
 from manuskript.ui.welcome import welcome
 from manuskript.ui import style as S
 
-try:
-    import enchant
-except ImportError:
-    enchant = None
-
 
 class settingsWindow(QWidget, Ui_Settings):
     def __init__(self, mainWindow):
@@ -76,11 +71,14 @@ class settingsWindow(QWidget, Ui_Settings):
         tr["English"] = ""
         tr["Arabic (Saudi Arabia)"] = "manuskript_ar_SA.qm"
         tr["Deutsch"] = "manuskript_de.qm"
+        tr["English (Great Britain)"] = "manuskript_en_GB.qm"
         tr["Español"] = "manuskript_es.qm"
+        tr["Persian"] = "manuskript_fa.qm"
         tr["Français"] = "manuskript_fr.qm"
         tr["Hungarian"] = "manuskript_hu.qm"
         tr["Indonesian"] = "manuskript_id.qm"
         tr["Italian"] = "manuskript_it.qm"
+        tr["Japanese"] = "manuskript_ja.qm"
         tr["Norwegian Bokmål"] = "manuskript_nb_NO.qm"
         tr["Dutch"] = "manuskript_nl.qm"
         tr["Polish"] = "manuskript_pl.qm"
@@ -88,8 +86,10 @@ class settingsWindow(QWidget, Ui_Settings):
         tr["Portuguese (Portugal)"] = "manuskript_pt_PT.qm"
         tr["Russian"] = "manuskript_ru.qm"
         tr["Svenska"] = "manuskript_sv.qm"
-        tr["Ukranian"] = "manuskript_uk.qm"
+        tr["Turkish"] = "manuskript_tr.qm"
+        tr["Ukrainian"] = "manuskript_uk.qm"
         tr["Chinese (Simplified)"] = "manuskript_zh_CN.qm"
+        tr["Chinese (Traditional)"] = "manuskript_zh_HANT.qm"
         self.translations = tr
 
         for name in tr:
